package com.qmth.themis.task.quartz;

import com.qmth.themis.business.service.TEExamService;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.task.config.DictionaryConfig;
import com.qmth.themis.task.service.QuartzService;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.quartz.QuartzJobBean;

import javax.annotation.Resource;
import java.util.*;

/**
 * @Description: 考试job
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/27
 */
public class ExamJob extends QuartzJobBean {
    private final static Logger log = LoggerFactory.getLogger(ExamJob.class);

    @Resource
    TEExamService teExamService;

    @Resource
    QuartzService quartzService;

    @Resource
    DictionaryConfig dictionaryConfig;

    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        log.info("exam_job进来了,context:{}", context);
        List<Map> teExamList = teExamService.getWaitingExamForJob();
        if (Objects.nonNull(teExamList) && teExamList.size() > 0) {
            for (int i = 0; i < teExamList.size(); i++) {
                Map map = teExamList.get(i);
                log.info("map:{}", JacksonUtil.parseJson(map));
                Integer forceFinish = Integer.parseInt(String.valueOf(map.get("forceFinish")));
                Date startTime = (Date) map.get("startTime");
                Date finishTime = (Date) map.get("finishTime");
                if (Objects.nonNull(forceFinish) && forceFinish.intValue() == 1) {//强制收卷
                    Long activityMaxDurationSeconds = Objects.nonNull(map.get("activityMaxDurationSeconds")) ? Long.parseLong(String.valueOf(map.get("activityMaxDurationSeconds"))) : null;
                    Long maxDurationSeconds = Objects.nonNull(map.get("maxDurationSeconds")) ? Long.parseLong(String.valueOf(map.get("maxDurationSeconds"))) : null;
                    Long finalMaxDurationSeconds = Objects.nonNull(activityMaxDurationSeconds) ? activityMaxDurationSeconds : maxDurationSeconds;
                    Calendar calendar = Calendar.getInstance();
                    if (Objects.nonNull(finalMaxDurationSeconds)) {
                        calendar.setTime(startTime);
                        calendar.add(Calendar.SECOND, activityMaxDurationSeconds.intValue());
                    } else {
                        calendar.setTime(finishTime);
                    }
//                    calendar.setTime(new Date());
                    int year = calendar.get(Calendar.YEAR);//获取年份
                    int month = calendar.get(Calendar.MONTH) + 1;//获取月份
                    int day = calendar.get(Calendar.DATE);//获取日
                    int hour = calendar.get(Calendar.HOUR_OF_DAY);//小时
                    int minute = calendar.get(Calendar.MINUTE);//分
                    int second = calendar.get(Calendar.SECOND);//秒
//                    String cron = (second + 30) + " " + (minute) + " " + hour + " " + day + " " + month + " ? " + year;
                    String cron = second + " " + (minute + 1) + " " + hour + " " + day + " " + month + " ? " + year;
                    log.info("cron:{}", cron);
                    String activityCode = String.valueOf(map.get("activityCode"));
                    //执行一次性延时任务
                    Map mapJob = new HashMap();
                    mapJob.put("name", activityCode);
                    quartzService.deleteJob(activityCode, dictionaryConfig.quartzConfigDomain().getExamActivityJobGroupName());
                    quartzService.addJob(ExamActivityJob.class, activityCode, dictionaryConfig.quartzConfigDomain().getExamActivityJobGroupName(), cron, mapJob);
                }
            }
        }
    }
}
